package board;

import java.awt.event.*;
import java.awt.*;
import java.util.*;



import world.Vector2D;
import world.World;

public class MainPanel extends Panel implements ButtonsInterface{

	//
	// buttons
	//
	public void initializeButtons(Buttons buttons){
		buttons.addButton("left");
		buttons.addButton("right");
		buttons.addButton("forward");
		buttons.addButton("backward");
		buttons.addButton("stop");
		buttons.addButton("reset");
	}
	public void buttonPressed(String label){
		if (label.equals("reset"))
			world.initialize();
		if (label.equals("left"))
			world.set_steeringwheel_angle(world.get_steeringwheel_angle()-Math.PI/16);
		if (label.equals("right"))
			world.set_steeringwheel_angle(world.get_steeringwheel_angle()+Math.PI/16);
		if (label.equals("forward"))
			world.set_drive_speed(5);
		if (label.equals("backward"))
			world.set_drive_speed(-5);
		if (label.equals("stop"))
			world.set_drive_speed(0);
		
	}	
	
	
	
	
	
	
	
	
	
	//
	// use this method for painting user interface elements
	//
	void paint_main(Graphics g){
		paint_feedback(g);
		paint_stroke(g);
		
		
	}
	
	
	
	//
	//	sample code : dragging
	//
	Vector2D start_point;
	Vector2D end_point;
	void paint_feedback(Graphics g){
		if (start_point == null || end_point == null)
			return;
		g.setColor(Color.red);
		g.drawLine((int)start_point.x, (int)start_point.y, (int)end_point.x, (int)end_point.y);
	}
	boolean start_dragging(Vector2D v){
		if (world.pick_robot(v)){
			start_point = end_point = v;
			return true;
		}
		return false;
	}
	void dragging(Vector2D v){
		end_point = v;
	}
	void finish_dragging(Vector2D v){
		start_point = end_point = null;
	}

	
	//
	//	sample code : stroke
	//
	ArrayList<Vector2D> stroke = new ArrayList<Vector2D>();
	public void start_drawing(Vector2D v){
		stroke.clear();
		drawing(v);
	}
	public void drawing(Vector2D v){
		stroke.add(v);
	}
	public void finish_drawing(){
		stroke.clear();
	}
	public void paint_stroke(Graphics g){
		g.setColor(Color.red);
		for(int i=0; i<stroke.size()-1; i++){
			Vector2D p = stroke.get(i);
			Vector2D q = stroke.get(i+1);
			g.drawLine((int)p.x, (int)p.y,(int) q.x, (int)q.y);
		}
	}
	
	
		
	
   
	//   
	// central event dispatcher
	//
	public int operation_status;
	public static final int NONE = 0;
	public static final int DRAGGING = 1;
	public static final int DRAWING = 2;
	public class EventDispatcher 
		extends MouseAdapter implements MouseMotionListener, KeyListener{

  		public void mousePressed(MouseEvent e) {
			Point p = e.getPoint();
			Vector2D v = new Vector2D(p);

			boolean right_button = (e.getModifiers() & e.BUTTON3_MASK) != 0;
			
			if (start_dragging(v)){
				operation_status = DRAGGING;
				return;
			}
			else{
				operation_status = DRAWING;
				start_drawing(v);
			}
		}
  		public void mouseDragged(MouseEvent e) {
			Point p = e.getPoint();
			Vector2D v = new Vector2D(p);
			
			switch(operation_status){
			case DRAGGING:
				dragging(v);
				break;
			case DRAWING:
				drawing(v);
				break;
			}
			
   		}
		public void mouseReleased(MouseEvent e) {
			Point p = e.getPoint();
			Vector2D v = new Vector2D(p);
			
			switch(operation_status){
			case DRAGGING:
				finish_dragging(v);
				break;
			case DRAWING:
				finish_drawing();
				break;
			}
		}
		public void mouseClicked(MouseEvent e){	}
  		public void mouseMoved(MouseEvent e) {}

		public void keyTyped(KeyEvent e){}
		public void keyReleased(KeyEvent e){}
		public void keyPressed(KeyEvent e){
			if (e.getKeyCode() == e.VK_UP){
				System.out.println("Up pressed");
			}
		}	
    }
	
	
	
	
	
	
	
	
	
	
	//
	// do not edit the code below this line ------------------
	//
	
	MainPanel(){
		EventDispatcher mouseDispatcher = new EventDispatcher();
		addMouseListener(mouseDispatcher);
		addMouseMotionListener(mouseDispatcher);
		addKeyListener(mouseDispatcher);
		
		this.setPreferredSize(new Dimension(600,600));
		
		world = new World();
		world.start(this);
	}
	static public World world;
	
	Image buffered_image = null;
	public void update(Graphics g) {
		paint(g);
	}
	public void paint(Graphics g) {

			//double buffering
			if (buffered_image == null)
				buffered_image = createImage(getSize().width, getSize().height);
			Graphics bg = buffered_image.getGraphics();

			// clear
			bg.setColor(Color.white);
			bg.fillRect(0,0, getSize().width, getSize().height);

			world.paint(bg);
			paint_main(bg);
			
			//double buffering
			bg.dispose();
      
			g.drawImage(buffered_image,0,0,this);

	}
	   public static Frame f;
		public static void main(String args[]) {
		f = new Frame("Main");

		f.addWindowListener(
		  new WindowAdapter(){
			public void windowClosing(WindowEvent e) {System.exit(0);}}
			);
		f.addKeyListener(
		  new KeyAdapter(){
		    public void keyPress(KeyEvent e) {
				if (e.getKeyCode() == e.VK_C)
					System.exit(0);}}
	        );

		Panel panel = new Panel();
		panel.setLayout(new BorderLayout());
		MainPanel mainPanel = new MainPanel();
		panel.add("Center", mainPanel);
		panel.add("South",new Buttons(mainPanel));
		

		f.add("Center", panel);
		f.setMenuBar(new MyMenuBar());
		f.doLayout();
		f.pack();
		f.show();

	
   }
	
}
